@@ -36,13 +36,6 @@ typedef struct pgpDigParams_s * pgpDigParams;
 typedef uint8_t pgpKeyID_t[8];
 typedef uint8_t pgpTime_t[4];
 
-/** \ingroup rpmpgp
- */
-typedef const struct pgpValTbl_s {
-    int val;
-    char const * const str;
-} * pgpValTbl;
- 
 /** \ingroup rpmpgp
  * 4.3. Packet Tags
  * 
@@ -251,14 +244,16 @@ typedef enum pgpCompressAlgo_e {
        4          - Reserved for double-width SHA (experimental)
        5          - MD2                                    "MD2"
        6          - Reserved for TIGER/192                 "TIGER192"
-       7          - Reserved for HAVAL (5 pass, 160-bit)
-       "HAVAL-5-160"
+       7          - Reserved for HAVAL (5 pass, 160-bit)    "HAVAL-5-160"
+       8          - SHA-256                                "SHA256"
+       9          - SHA-384                                "SHA384"
+       10         - SHA-512                                "SHA512"
+       11         - SHA-224                                "SHA224"
        100 to 110 - Private/Experimental algorithm.
 \endverbatim
  *
  * Implementations MUST implement SHA-1. Implementations SHOULD
  * implement MD5.
- * @todo Add SHA256.
  */
 typedef enum pgpHashAlgo_e {
     PGPHASHALGO_MD5		=  1,	/*!< MD5 */
@@ -270,6 +265,7 @@ typedef enum pgpHashAlgo_e {
     PGPHASHALGO_SHA256		=  8,	/*!< SHA256 */
     PGPHASHALGO_SHA384		=  9,	/*!< SHA384 */
     PGPHASHALGO_SHA512		= 10,	/*!< SHA512 */
+    PGPHASHALGO_SHA224		= 11,	/*!< SHA224 */
 } pgpHashAlgo;
 
 /** \ingroup rpmpgp
@@ -920,13 +916,35 @@ typedef enum pgpArmorKey_e {
     PGPARMORKEY_CHARSET		= 5  /*!< Charset: */
 } pgpArmorKey;
 
+typedef enum pgpValType_e {
+    PGPVAL_TAG			= 1,
+    PGPVAL_ARMORBLOCK		= 2,
+    PGPVAL_ARMORKEY		= 3,
+    PGPVAL_SIGTYPE		= 4,
+    PGPVAL_SUBTYPE		= 5,
+    PGPVAL_PUBKEYALGO		= 6,
+    PGPVAL_SYMKEYALGO		= 7,
+    PGPVAL_COMPRESSALGO		= 8,
+    PGPVAL_HASHALGO		= 9,
+    PGPVAL_SERVERPREFS		= 10,
+} pgpValType;
+
 /** \ingroup rpmpgp
  * Bit(s) to control digest operation.
  */
-typedef enum rpmDigestFlags_e {
+enum rpmDigestFlags_e {
     RPMDIGEST_NONE	= 0
-} rpmDigestFlags;
+};
 
+typedef rpmFlags rpmDigestFlags;
+
+/** \ingroup rpmpgp
+ * Return string representation of am OpenPGP value.
+ * @param type		type of value
+ * @param val		byte value to lookup
+ * @return		string value of byte
+ */
+const char * pgpValString(pgpValType type, uint8_t val);
 
 /** \ingroup rpmpgp
  * Return (native-endian) integer from big-endian representation.
@@ -944,27 +962,6 @@ unsigned int pgpGrab(const uint8_t *s, size_t nbytes)
     return i;
 }
 
-/** \ingroup rpmpgp
- * Return length of an OpenPGP packet.
- * @param s		pointer to packet
- * @retval *lenp	no. of bytes in packet
- * @return		no. of bytes in length prefix
- */
-static inline
-size_t pgpLen(const uint8_t *s, size_t * lenp)
-{
-    if (*s < 192) {
-	(*lenp) = *s++;
-	return 1;
-    } else if (*s < 255) {
-	(*lenp) = ((((unsigned)s[0]) - 192) << 8) + s[1] + 192;
-	return 2;
-    } else {
-	(*lenp) = pgpGrab(s+1, (size_t) 4);
-	return 5;
-    }
-}
-
 /** \ingroup rpmpgp
  * Return hex formatted representation of bytes.
  * @param p		bytes
@@ -975,24 +972,47 @@ char * pgpHexStr(const uint8_t *p, size_t plen);
 
 /** \ingroup rpmpgp
  * Calculate OpenPGP public key fingerprint.
- * @todo V3 non-RSA public keys not implemented.
  * @param pkt		OpenPGP packet (i.e. PGPTAG_PUBLIC_KEY)
  * @param pktlen	OpenPGP packet length (no. of bytes)
- * @retval keyid	public key fingerprint
- * @return		0 on sucess, else -1
+ * @retval fp		public key fingerprint
+ * @retval fplen	public key fingerprint length
+ * @return		0 on success, else -1
  */
 int pgpPubkeyFingerprint(const uint8_t * pkt, size_t pktlen,
-		pgpKeyID_t keyid);
+			 uint8_t **fp, size_t *fplen);
 
 /** \ingroup rpmpgp
-* Extract OpenPGP public key fingerprint from base64 encoded packet.
-* @todo V3 non-RSA public keys not implemented.
-* @param b64pkt       	base64 encoded openpgp packet
-* @retval keyid		public key fingerprint
-* @return             	8 (no. of bytes) on success, < 0 on error
-*/
-int pgpExtractPubkeyFingerprint(const char * b64pkt, pgpKeyID_t keyid);
+ * Calculate OpenPGP public key Key ID
+ * @param pkt		OpenPGP packet (i.e. PGPTAG_PUBLIC_KEY)
+ * @param pktlen	OpenPGP packet length (no. of bytes)
+ * @retval keyid	public key Key ID
+ * @return		0 on success, else -1
+ */
+int pgpPubkeyKeyID(const uint8_t * pkt, size_t pktlen, pgpKeyID_t keyid);
 
+/** \ingroup rpmpgp
+ * Parse a OpenPGP packet(s).
+ * @param pkts		OpenPGP packet(s)
+ * @param pktlen	OpenPGP packet(s) length (no. of bytes)
+ * @param pkttype	Expected packet type (signature/key) or 0 for any
+ * @retval ret		signature/pubkey packet parameters on success (alloced)
+ * @return		-1 on error, 0 on success
+ */
+int pgpPrtParams(const uint8_t *pkts, size_t pktlen, unsigned int pkttype,
+		 pgpDigParams * ret);
+
+/** \ingroup rpmpgp
+ * Parse subkey parameters from OpenPGP packet(s).
+ * @param pkts		OpenPGP packet(s)
+ * @param pktlen	OpenPGP packet(s) length (no. of bytes)
+ * @param mainkey	parameters of main key
+ * @param subkeys	array of subkey parameters (alloced)
+ * @param subkeysCount	count of subkeys
+ * @return		-1 on error, 0 on success
+ */
+int pgpPrtParamsSubkeys(const uint8_t *pkts, size_t pktlen,
+			pgpDigParams mainkey, pgpDigParams **subkeys,
+			int *subkeysCount);
 /** \ingroup rpmpgp
  * Print/parse a OpenPGP packet(s).
  * @param pkts		OpenPGP packet(s)
@@ -1021,6 +1041,20 @@ pgpArmor pgpReadPkts(const char * fn, uint8_t ** pkt, size_t * pktlen);
  */
 pgpArmor pgpParsePkts(const char *armor, uint8_t ** pkt, size_t * pktlen);
 
+/** \ingroup rpmpgp
+ * Return a length of the first public key certificate in a buffer given
+ * by pkts that contains one or more certificates. A public key certificate
+ * consits of packets like Public key packet, User ID packet and so on.
+ * In a buffer every certificate starts with Public key packet and it ends
+ * with the start of the next certificate or with the end of the buffer.
+ *
+ * @param pkts		pointer to a buffer with certificates
+ * @param pktslen	length of the buffer with certificates
+ * @param certlen	length of the first certificate in the buffer
+ * @return		0 on success
+ */
+int pgpPubKeyCertLen(const uint8_t *pkts, size_t pktslen, size_t *certlen);
+
 /** \ingroup rpmpgp
  * Wrap a OpenPGP packets in ascii armor for transport.
  * @param atype		type of armor
@@ -1049,8 +1083,50 @@ void pgpCleanDig(pgpDig dig);
  */
 pgpDig pgpFreeDig(pgpDig dig);
 
+/** \ingroup rpmpgp
+ * Retrieve parameters for parsed OpenPGP packet(s).
+ * @param dig		container
+ * @param pkttype	type of params to retrieve (signature / pubkey)
+ * @return		pointer to OpenPGP parameters, NULL on error/not found
+ */
+pgpDigParams pgpDigGetParams(pgpDig dig, unsigned int pkttype);
+
+/** \ingroup rpmpgp
+ * Compare OpenPGP packet parameters
+ * param p1		1st parameter container
+ * param p2		2nd parameter container
+ * return		1 if the parameters differ, 0 otherwise
+ */
+int pgpDigParamsCmp(pgpDigParams p1, pgpDigParams p2);
+
+/** \ingroup rpmpgp
+ * Retrieve OpenPGP algorithm parameters
+ * param digp		parameter container
+ * param algotype	PGPVAL_HASHALGO / PGPVAL_PUBKEYALGO
+ * return		algorithm value, 0 on error
+ */
+unsigned int pgpDigParamsAlgo(pgpDigParams digp, unsigned int algotype);
+
+/** \ingroup rpmpgp
+ * Destroy parsed OpenPGP packet parameter(s).
+ * @param digp		parameter container
+ * @return		NULL always
+ */
+pgpDigParams pgpDigParamsFree(pgpDigParams digp);
+
 /** \ingroup rpmpgp
  * Verify a PGP signature.
+ * @param key		public key
+ * @param sig		signature
+ * @param hashctx	digest context
+ * @return 		RPMRC_OK on success 
+ */
+rpmRC pgpVerifySignature(pgpDigParams key, pgpDigParams sig, DIGEST_CTX hashctx);
+
+/** \ingroup rpmpgp
+ * Verify a PGP signature.
+ * @deprecated		use pgpVerifySignature() instead
+ *
  * @param dig		container
  * @param hashctx	digest context
  * @return 		RPMRC_OK on success 
@@ -1090,7 +1166,7 @@ DIGEST_CTX rpmDigestDup(DIGEST_CTX octx);
  * @param hashalgo	type of digest
  * @return		digest length, zero on invalid algorithm
  */
-size_t rpmDigestLength(pgpHashAlgo hashalgo);
+size_t rpmDigestLength(int hashalgo);
 
 /** \ingroup rpmpgp
  * Initialize digest.
@@ -1099,7 +1175,7 @@ size_t rpmDigestLength(pgpHashAlgo hashalgo);
  * @param flags		bit(s) to control digest operation
  * @return		digest context
  */
-DIGEST_CTX rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags);
+DIGEST_CTX rpmDigestInit(int hashalgo, rpmDigestFlags flags);
 
 /** \ingroup rpmpgp
  * Update context with next plain text buffer.
@@ -1139,15 +1215,27 @@ rpmDigestBundle rpmDigestBundleNew(void);
 rpmDigestBundle rpmDigestBundleFree(rpmDigestBundle bundle);
 
 /** \ingroup rpmpgp
- * Add a new type of digest to a bundle.
+ * Add a new type of digest to a bundle. Same as calling
+ * rpmDigestBundleAddID() with algo == id value.
  * @param bundle	digest bundle
  * @param algo		type of digest
  * @param flags		bit(s) to control digest operation
  * @return		0 on success
  */
-int rpmDigestBundleAdd(rpmDigestBundle bundle, pgpHashAlgo algo,
+int rpmDigestBundleAdd(rpmDigestBundle bundle, int algo,
 			rpmDigestFlags flags);
 
+/** \ingroup rpmpgp
+ * Add a new type of digest to a bundle.
+ * @param bundle	digest bundle
+ * @param algo		type of digest
+ * @param id		id of digest (arbitrary, must be > 0)
+ * @param flags		bit(s) to control digest operation
+ * @return		0 on success
+ */
+int rpmDigestBundleAddID(rpmDigestBundle bundle, int algo, int id,
+			 rpmDigestFlags flags);
+
 /** \ingroup rpmpgp
  * Update contexts within bundle with next plain text buffer.
  * @param bundle	digest bundle
@@ -1161,22 +1249,22 @@ int rpmDigestBundleUpdate(rpmDigestBundle bundle, const void *data, size_t len);
  * Return digest from a bundle and destroy context, see rpmDigestFinal().
  *
  * @param bundle	digest bundle
- * @param algo		type of digest to return
+ * @param id		id of digest to return
  * @retval datap	address of returned digest
  * @retval lenp		address of digest length
  * @param asAscii	return digest as ascii string?
  * @return		0 on success
  */
-int rpmDigestBundleFinal(rpmDigestBundle bundle,
-	 pgpHashAlgo algo, void ** datap, size_t * lenp, int asAscii);
+int rpmDigestBundleFinal(rpmDigestBundle bundle, int id,
+			 void ** datap, size_t * lenp, int asAscii);
 
 /** \ingroup rpmpgp
  * Duplicate a digest context from a bundle.
  * @param bundle	digest bundle
- * @param algo		type of digest to dup
+ * @param id		id of digest to dup
  * @return		duplicated digest context
  */
-DIGEST_CTX rpmDigestBundleDupCtx(rpmDigestBundle bundle, pgpHashAlgo algo);
+DIGEST_CTX rpmDigestBundleDupCtx(rpmDigestBundle bundle, int id);
 
 #ifdef __cplusplus
 }
